package flutter.curiosity

import android.annotation.SuppressLint
import android.app.Activity
import android.app.ActivityManager
import android.content.Context
import android.content.Intent
import android.content.pm.ApplicationInfo
import android.content.pm.PackageInfo
import android.content.res.Resources
import android.location.LocationManager
import android.net.Uri
import android.os.Build
import android.os.Environment
import android.os.Process
import android.provider.Settings
import androidx.annotation.RequiresApi
import androidx.collection.ArrayMap
import androidx.core.content.FileProvider
import io.flutter.plugin.common.MethodCall
import java.io.File
import kotlin.collections.set
import kotlin.system.exitProcess


object NativeTools {


    private fun getBarHeight(barName: String?, context: Context): Float {
        val resources: Resources = context.resources
        val resourceId = resources.getIdentifier(barName, "dimen", "android")
        return resources.getDimensionPixelSize(resourceId).toFloat()
    }

    /**
     * 获取app 信息
     */
    fun getAppInfo(context: Context): Map<String, Any?> {
        val pm = context.packageManager
        val info = context.packageManager.getPackageInfo(context.packageName, 0)
        return mapOf<String, Any?>(
                "sdkVersion" to Build.VERSION.SDK_INT,
                "versionCode" to if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.P) info.longVersionCode else info.versionCode,
                "appName" to info.applicationInfo.loadLabel(pm).toString(),
                "packageName" to info.packageName,
                "versionName" to info.versionName,
                "firstInstallTime" to info.firstInstallTime,
                "lastUpdateTime" to info.lastUpdateTime,
                "statusBarHeight" to getBarHeight("status_bar_height", context),
                "navigationBarHeight" to getBarHeight("navigation_bar_height", context)
        )
    }

    /**
     * 获取app路径
     */
    fun getAppPath(context: Context): Map<String, Any?> {
        return mapOf<String, Any?>(
                "filesDir" to context.filesDir.absolutePath,
                "cacheDir" to context.cacheDir.absolutePath,
                "externalCacheDir" to context.externalCacheDir?.absolutePath,
                "externalFilesDir" to context.getExternalFilesDir(null)?.absolutePath,
                "externalStorageDirectory" to Tools.getExternalDirectory(null, context),
                "directoryDCIM" to Tools.getExternalDirectory(Environment.DIRECTORY_DCIM, context),
                "directoryDownloads" to Tools.getExternalDirectory(
                        Environment.DIRECTORY_DOWNLOADS,
                        context
                ),
                "directoryMovies" to Tools.getExternalDirectory(
                        Environment.DIRECTORY_MOVIES,
                        context
                ),
                "directoryMusic" to
                        Tools.getExternalDirectory(Environment.DIRECTORY_MUSIC, context),
                "directoryPictures" to Tools.getExternalDirectory(
                        Environment.DIRECTORY_PICTURES,
                        context
                ),
                "directoryAlarms" to Tools.getExternalDirectory(
                        Environment.DIRECTORY_ALARMS,
                        context
                ),
                "directoryDocuments" to Tools.getExternalDirectory(
                        Environment.DIRECTORY_DOCUMENTS,
                        context
                ),
                "directoryNotifications" to Tools.getExternalDirectory(
                        Environment.DIRECTORY_NOTIFICATIONS,
                        context
                ),
                "directoryRINGTONES" to Tools.getExternalDirectory(
                        Environment.DIRECTORY_RINGTONES,
                        context
                ),
                "directoryPODCASTS" to Tools.getExternalDirectory(
                        Environment.DIRECTORY_PODCASTS,
                        context
                )
        )
    }

    /**
     * 获取设备信息
     */
    @SuppressLint("HardwareIds")
    fun getDeviceInfo(context: Context): Map<String, Any?> {
        val map: MutableMap<String, Any?> = HashMap()
        map["board"] = Build.BOARD
        map["bootloader"] = Build.BOOTLOADER
        map["brand"] = Build.BRAND
        map["device"] = Build.DEVICE
        map["deviceId"] = UDIDHelper.getUniqueDeviceId(context)
        map["generateDeviceId"] = UDIDHelper.generateUniqueDeviceId()
        map["display"] = Build.DISPLAY
        map["fingerprint"] = Build.FINGERPRINT
        map["hardware"] = Build.HARDWARE
        map["host"] = Build.HOST
        map["id"] = Build.ID
        map["manufacturer"] = Build.MANUFACTURER
        map["model"] = Build.MODEL
        map["product"] = Build.PRODUCT
        map["tags"] = Build.TAGS
        map["type"] = Build.TYPE
        map["isEmulator"] = !Tools.isEmulator()
        map["androidId"] = Settings.Secure.getString(
                context.contentResolver,
                Settings.Secure.ANDROID_ID
        )
        map["isDeviceRoot"] = Tools.isDeviceRoot()
        val version: MutableMap<String, Any> = HashMap()
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
            version["baseOS"] = Build.VERSION.BASE_OS
            version["previewSdkInt"] = Build.VERSION.PREVIEW_SDK_INT
            version["securityPatch"] = Build.VERSION.SECURITY_PATCH
        }
        version["codename"] = Build.VERSION.CODENAME
        version["incremental"] = Build.VERSION.INCREMENTAL
        version["release"] = Build.VERSION.RELEASE
        version["sdkInt"] = Build.VERSION.SDK_INT
        map["version"] = version
        return map
    }

    /**
     * 获取应用列表
     */
    fun getInstalledApp(context: Context): ArrayList<MutableMap<String, Any>> {
        val list: ArrayList<MutableMap<String, Any>> = ArrayList()
        val pm = context.packageManager
        val packages: MutableList<PackageInfo> =
                pm.getInstalledPackages(0) // 获取所有已安装程序的包信息
        for (packageInfo in packages) {
            val info: MutableMap<String, Any> = ArrayMap()
            info["isSystemApp"] = (packageInfo.applicationInfo.flags and
                    ApplicationInfo.FLAG_SYSTEM) != 0
            info["versionName"] = packageInfo.versionName
            info["appName"] = packageInfo.applicationInfo.loadLabel(pm)
            info["packageName"] = packageInfo.packageName
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.P) {
                info["versionCode"] = packageInfo.longVersionCode
            } else {
                info["versionCode"] = packageInfo.versionCode
            }
            info["lastUpdateTime"] = packageInfo.lastUpdateTime
            list.add(info)
        }
        return list
    }


    /**
     * 安装apk
     */
    fun getInstallAppIntent(context: Context, call: MethodCall): Intent? {
        //安装
        val apkPath = call.arguments as String
        val file = File(apkPath)
        if (!file.exists()) {
            return null
        }
        val intent = Intent(Intent.ACTION_VIEW)
        //版本在7.0以上是不能直接通过uri访问的
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.N) {
            //参数1 上下文, 参数2 Provider主机地址 和配置文件中保持一致   参数3  共享的文件
            val apkUri = FileProvider.getUriForFile(
                    context,
                    context.packageName + ".provider",
                    file
            )
            //添加这一句表示对目标应用临时授权该Uri所代表的文件
            intent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION)
            intent.setDataAndType(
                    apkUri,
                    "application/vnd.android.package-archive"
            )
        } else {
            intent.setDataAndType(
                    Uri.fromFile(file),
                    "application/vnd.android.package-archive"
            )
        }
        return intent
    }

    @RequiresApi(Build.VERSION_CODES.O)
    fun getRequestPackageInstallsIntent(context: Context): Intent {
        val packageURI = Uri.parse("package:" + context.packageName)
        return Intent(
                Settings.ACTION_MANAGE_UNKNOWN_APP_SOURCES,
                packageURI
        )
    }

    /**
     * 退出app
     */
    fun exitApp(activity: Activity) {
        //杀死进程，否则就算退出App，App处于空进程并未销毁，再次打开也不会初始化Application
        val manager = activity.getSystemService(Context.ACTIVITY_SERVICE) as ActivityManager
        manager.runningAppProcesses.forEach { processInfo ->
            if (processInfo.pid != Process.myPid()) {
                Process.killProcess(processInfo.pid)
            }
        }
//        Process.killProcess(Process.myPid())
        exitProcess(0)
    }

    /**
     * 判断手机是否安装某个应用
     *
     * @return true：安装，false：未安装
     */
    fun isInstallApp(context: Context, call: MethodCall): Boolean {
        val packageName = call.arguments
        val packages: MutableList<PackageInfo> =
                context.packageManager.getInstalledPackages(0) // 获取所有已安装程序的包信息
        for (packageInfo in packages) {
            if (packageName == packageInfo.packageName) {
                return true
            }
        }
        return false
    }

    /**
     * 跳转至应用商店
     */
    fun getAppMarketIntent(call: MethodCall): Intent? {
        val packageName = call.argument<String>("packageName")
        if (packageName == null || packageName.isEmpty()) return null
        val marketPackageName = call.argument<String>("marketPackageName")
        val uri = Uri.parse("market://details?id=$packageName")
        val intent = Intent(Intent.ACTION_VIEW, uri)
        intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
        if (marketPackageName != null && marketPackageName != "") {
            intent.setPackage(marketPackageName)
        }
        return intent
    }


    /**
     * 跳转到系统自带的设置
     */
    fun getSystemSettingIntent(activity: Activity, call: MethodCall): Intent {
        val settingType = call.arguments as String
        var intent = Intent()
        if (settingType == "wifi") {
            intent = Intent(Settings.ACTION_WIFI_SETTINGS)
        } else if (settingType == "wifiIp") {
            intent = Intent(Settings.ACTION_WIFI_IP_SETTINGS)
        } else if (settingType == "location") {
            intent = Intent(Settings.ACTION_LOCATION_SOURCE_SETTINGS)
        } else if (settingType == "security") {
            intent = Intent(Settings.ACTION_SECURITY_SETTINGS)
        } else if (settingType == "passwordSecurity") {
            intent = Intent(Settings.ACTION_SECURITY_SETTINGS)
        } else if (settingType == "cellularNetwork") {
            intent = Intent(Settings.ACTION_DATA_ROAMING_SETTINGS)
        } else if (settingType == "bluetooth") {
            intent = Intent(Settings.ACTION_BLUETOOTH_SETTINGS)
        } else if (settingType == "time") {
            intent = Intent(Settings.ACTION_DATE_SETTINGS)
        } else if (settingType == "displayBrightness") {
            intent = Intent(Settings.ACTION_DISPLAY_SETTINGS)
        } else if (settingType == "notification") {
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
                intent = Intent(Settings.ACTION_APP_NOTIFICATION_SETTINGS)
                        .putExtra(
                                Settings.EXTRA_APP_PACKAGE,
                                activity.packageName
                        )
            } else if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M && Build
                            .VERSION.SDK_INT < Build.VERSION_CODES.O
            ) {
                intent = Intent(
                        Settings
                                .ACTION_NOTIFICATION_POLICY_ACCESS_SETTINGS
                )
            }
        } else if (settingType == "soundVibration") {
            intent = Intent(Settings.ACTION_SOUND_SETTINGS)
        } else if (settingType == "internalStorage") {
            intent = Intent(Settings.ACTION_INTERNAL_STORAGE_SETTINGS)
        } else if (settingType == "battery") {
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
                intent = Intent(
                        Settings
                                .ACTION_IGNORE_BATTERY_OPTIMIZATION_SETTINGS
                )
            }
        } else if (settingType == "nfc") {
            intent = Intent(Settings.ACTION_NFC_SETTINGS)
        } else if (settingType == "localeLanguage") {
            intent = Intent(Settings.ACTION_LOCALE_SETTINGS)
        } else if (settingType == "deviceInfo") {
            intent = Intent(Settings.ACTION_DEVICE_INFO_SETTINGS)
        } else if (settingType == "applicationDevelopment") {
            intent =
                    Intent(Settings.ACTION_APPLICATION_DEVELOPMENT_SETTINGS)
        } else if (settingType == "networkOperator") {
            intent = Intent(Settings.ACTION_NETWORK_OPERATOR_SETTINGS)
        } else if (settingType == "addAccount") {
            intent = Intent(Settings.ACTION_ADD_ACCOUNT)
        } else if (settingType == "dataRoaming") {
            intent = Intent(Settings.ACTION_DATA_ROAMING_SETTINGS)
        } else if (settingType == "airplaneMode") {
            intent = Intent(Settings.ACTION_AIRPLANE_MODE_SETTINGS)
        } else if (settingType == "setting") {
            intent = Intent(Settings.ACTION_SETTINGS)
        } else {
            intent.action = Settings.ACTION_APPLICATION_DETAILS_SETTINGS
            intent.addCategory(Intent.CATEGORY_DEFAULT)
            intent.data = Uri.parse("package:" + activity.packageName)
            intent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)
            intent.addFlags(Intent.FLAG_ACTIVITY_NO_HISTORY)
            intent.addFlags(Intent.FLAG_ACTIVITY_EXCLUDE_FROM_RECENTS)
            intent = Intent(Settings.ACTION_APPLICATION_DETAILS_SETTINGS)
        }
        return intent
    }


    /**
     * 判断GPS是否开启，GPS或者AGPS开启一个就认为是开启的
     * @return true 表示开启
     */
    fun getGPSStatus(context: Context): Boolean {
        val locationManager: LocationManager =
                context.getSystemService(Context.LOCATION_SERVICE) as LocationManager
        // 通过GPS卫星定位，定位级别可以精确到街（通过24颗卫星定位，在室外和空旷的地方定位准确、速度快）
        val gps: Boolean =
                locationManager.isProviderEnabled(LocationManager.GPS_PROVIDER)
        // 通过WLAN或移动网络(3G/2G)确定的位置（也称作AGPS，辅助GPS定位。主要用于在室内或遮盖物（建筑群或茂密的深林等）密集的地方定位）
        val network: Boolean =
                locationManager.isProviderEnabled(LocationManager.NETWORK_PROVIDER)
        return gps || network
    }

}